package org.eclipse.jdt.internal.compiler.apt.dispatch;

package .tmp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.Processor;
import javax.tools.StandardLocation;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseAnnotationProcessorManager;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseProcessingEnvImpl;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BatchProcessingEnvImpl;
import org.eclipse.jdt.internal.compiler.apt.dispatch.ProcessorInfo;
import org.eclipse.jdt.internal.compiler.batch.Main;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;

public class BatchAnnotationProcessorManager extends BaseAnnotationProcessorManager {
  private List<Processor> _setProcessors = null;
  
  private Iterator<Processor> _setProcessorIter = null;
  
  private List<String> _commandLineProcessors;
  
  private Iterator<String> _commandLineProcessorIter = null;
  
  private ServiceLoader<Processor> _serviceLoader = null;
  
  private Iterator<Processor> _serviceLoaderIter;
  
  private ClassLoader _procLoader;
  
  private static final boolean VERBOSE_PROCESSOR_DISCOVERY = true;
  
  private boolean _printProcessorDiscovery = false;
  
  public void configure(Object batchCompiler, String[] commandLineArguments) {
    if (this._processingEnv != null)
      throw new IllegalStateException(
          "Calling configure() more than once on an AnnotationProcessorManager is not supported"); 
    BatchProcessingEnvImpl processingEnv = new BatchProcessingEnvImpl(this, (Main)batchCompiler, commandLineArguments);
    this._processingEnv = (BaseProcessingEnvImpl)processingEnv;
    this._procLoader = processingEnv.getFileManager().getClassLoader(StandardLocation.ANNOTATION_PROCESSOR_PATH);
    parseCommandLine(commandLineArguments);
    this._round = 0;
  }
  
  private void parseCommandLine(String[] commandLineArguments) {
    List<String> commandLineProcessors = null;
    for (int i = 0; i < commandLineArguments.length; i++) {
      String option = commandLineArguments[i];
      if ("-XprintProcessorInfo".equals(option)) {
        this._printProcessorInfo = true;
        this._printProcessorDiscovery = true;
      } else if ("-XprintRounds".equals(option)) {
        this._printRounds = true;
      } else if ("-processor".equals(option)) {
        commandLineProcessors = new ArrayList<>();
        String procs = commandLineArguments[++i];
        byte b;
        int j;
        String[] arrayOfString;
        for (j = (arrayOfString = procs.split(",")).length, b = 0; b < j; ) {
          String proc = arrayOfString[b];
          commandLineProcessors.add(proc);
          b++;
        } 
        break;
      } 
    } 
    this._commandLineProcessors = commandLineProcessors;
    if (this._commandLineProcessors != null)
      this._commandLineProcessorIter = this._commandLineProcessors.iterator(); 
  }
  
  public ProcessorInfo discoverNextProcessor() {
    if (this._setProcessors != null) {
      if (this._setProcessorIter.hasNext()) {
        Processor p = this._setProcessorIter.next();
        p.init((ProcessingEnvironment)this._processingEnv);
        ProcessorInfo pi = new ProcessorInfo(p);
        this._processors.add(pi);
        if (this._printProcessorDiscovery && this._out != null)
          this._out.println("API specified processor: " + pi); 
        return pi;
      } 
      return null;
    } 
    if (this._commandLineProcessors != null) {
      if (this._commandLineProcessorIter.hasNext()) {
        String proc = this._commandLineProcessorIter.next();
        try {
          Class<?> clazz = this._procLoader.loadClass(proc);
          Object o = clazz.newInstance();
          Processor p = (Processor)o;
          p.init((ProcessingEnvironment)this._processingEnv);
          ProcessorInfo pi = new ProcessorInfo(p);
          this._processors.add(pi);
          if (this._printProcessorDiscovery && this._out != null)
            this._out.println("Command line specified processor: " + pi); 
          return pi;
        } catch (Exception e) {
          throw new AbortCompilation(null, e);
        } 
      } 
      return null;
    } 
    if (this._serviceLoader == null) {
      this._serviceLoader = ServiceLoader.load(Processor.class, this._procLoader);
      this._serviceLoaderIter = this._serviceLoader.iterator();
    } 
    try {
      if (this._serviceLoaderIter.hasNext()) {
        Processor p = this._serviceLoaderIter.next();
        p.init((ProcessingEnvironment)this._processingEnv);
        ProcessorInfo pi = new ProcessorInfo(p);
        this._processors.add(pi);
        if (this._printProcessorDiscovery && this._out != null) {
          StringBuilder sb = new StringBuilder();
          sb.append("Discovered processor service ");
          sb.append(pi);
          sb.append("\n  supporting ");
          sb.append(pi.getSupportedAnnotationTypesAsString());
          sb.append("\n  in ");
          sb.append(getProcessorLocation(p));
          this._out.println(sb.toString());
        } 
        return pi;
      } 
    } catch (ServiceConfigurationError e) {
      throw new AbortCompilation(null, e);
    } 
    return null;
  }
  
  private String getProcessorLocation(Processor p) {
    boolean isMember = false;
    Class<?> outerClass = p.getClass();
    StringBuilder innerName = new StringBuilder();
    while (outerClass.isMemberClass()) {
      innerName.insert(0, outerClass.getSimpleName());
      innerName.insert(0, '$');
      isMember = true;
      outerClass = outerClass.getEnclosingClass();
    } 
    String path = outerClass.getName();
    path = path.replace('.', '/');
    if (isMember)
      path = String.valueOf(path) + innerName; 
    path = String.valueOf(path) + ".class";
    String location = this._procLoader.getResource(path).toString();
    if (location.endsWith(path))
      location = location.substring(0, location.length() - path.length()); 
    return location;
  }
  
  public void reportProcessorException(Processor p, Exception e) {
    throw new AbortCompilation(null, e);
  }
  
  public void setProcessors(Object[] processors) {
    if (!this._isFirstRound)
      throw new IllegalStateException("setProcessors() cannot be called after processing has begun"); 
    this._setProcessors = new ArrayList<>(processors.length);
    byte b;
    int i;
    Object[] arrayOfObject;
    for (i = (arrayOfObject = processors).length, b = 0; b < i; ) {
      Object o = arrayOfObject[b];
      Processor p = (Processor)o;
      this._setProcessors.add(p);
      b++;
    } 
    this._setProcessorIter = this._setProcessors.iterator();
    this._commandLineProcessors = null;
    this._commandLineProcessorIter = null;
  }
}
